package ch.loosli.protrading.signalLoader;

import java.util.ArrayList;
import java.util.Date;

/**
 * 
 * @author Roman Loosli
 * @since Version 1.0
 * @version 1.0
 *
 */
public class Equity implements Comparable<Equity>{
	
	private ArrayList<DayRecord>dayrecords = new ArrayList<DayRecord>();
	private String ticker;
	private String fullname;
	private Signal signal;
	private Date actualSignalStart;
	
	
	/**
	 * Initializes local variables ticker and fullname.
	 * @param ticker
	 * @param fullname
	 */
	public Equity(String ticker, String fullname) {
		this.ticker = ticker;
		this.fullname = fullname;
		this.signal = null;
		this.actualSignalStart = null;
	}
	
	
	/**
	 * Initializes local variable ticker.
	 * @param ticker
	 */
	public Equity(String ticker) {
		this(ticker,null);
	}
	
	
	
	
	/**
	 * @return the dayrecords
	 */
	public ArrayList<DayRecord> getDayrecords() {
		return dayrecords;
	}
	
	
	/**
	 * @return the ticker
	 * @since version 1.0
	 */
	public String getTicker() {
		return ticker;
	}
	
	
	/**
	 * @return the fullname
	 */
	public String getFullname() {
		return fullname;
	}
	
	
	
	/**
	 * Gets the last added DayRecord and returns its close rate.
	 * @return Last added close rate of this equity.
	 */
	public double getLastRate() {
		return this.dayrecords.get(this.dayrecords.size()-1).getClose();
	}
	
	
	public Signal getSignal() {
		return signal;
	}


	public void setSignal(Signal signal) {
		this.signal = signal;
		this.actualSignalStart = new Date();
	}


	/**
	 * @return the actualSignalStart
	 */
	public Date getActualSignalStart() {
		return actualSignalStart;
	}


	/**
	 * Get Highest Close-Rate from a Period.
	 * @param period Size of period in days.
	 * @param daysBack Days Back, first day after the period ended.
	 * @return The Highest Close Rate in that Period. -1 if period does not exist in data.
	 */
	public double getHighestCloseFromPeriod(int period, int daysBack) {
		double highest = -1;
		if(period + daysBack > dayrecords.size() || period == 0) {
			return highest;
		}
		
		for(int i = dayrecords.size() - (daysBack + period); i < dayrecords.size() - daysBack; i++) {
			if(dayrecords.get(i).getClose() > highest) {
				highest = dayrecords.get(i).getClose();
			}
		}
		return highest;
	}
	
	
	/**
	 * Get Lowest Close-Rate from a Period.
	 * @param period size of period in days.
	 * @param daysBack daysBack Days back, first day after the period ended.
	 * @return The Highest close rate in that period. -1 if period does not exist in data.
	 */
	public double getLowestCloseFromPeriod(int period, int daysBack) {
		double lowest = -1;
		if(period + daysBack > dayrecords.size() || period == 0) {
			return lowest;
		}
		
		for(int i = dayrecords.size() - (daysBack + period); i < dayrecords.size() - daysBack; i++) {
			if(dayrecords.get(i).getClose() < lowest || lowest == -1) {
				lowest = dayrecords.get(i).getClose();
			}
		}
		return lowest;
	}
	
	
	
	public void updateSignal() {
		this.setSignal(SignalChooser.chooseSignal(this));
	}
	



	@Override
	public int compareTo(Equity o) {
		if(this.getLastRate() == o.getLastRate()) {
			return 0;			
		}
		else if(this.getLastRate() > o.getLastRate()) {
			return 1;
		}
		else {
			return -1;
		}
	}
	
	
}
