package ch.loosli.protrading.signalLoader;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.time.LocalDate;
import java.util.ArrayList;


/**
 * 
 * @author Roman Loosli
 * @version 1.0
 * @since Version 1.0
 *
 */
public class QuoteFileParser {

	private static BufferedReader reader;
	
	
	/**
	 * Adds DayRecords to Equity from CSV File.
	 * @param equity Equity on which Dayrecords should be added.
	 * @param csv CSV file in which the Dayrecords should be parsed.
	 * @param daysBack How many days back the parsing should start.
	 * @return true if successfull, false if not.
	 */
	public static boolean addDayRecordsFromFile(Equity equity, InputStream csv, int daysBack) {
		ArrayList<DayRecord> temp = new ArrayList<DayRecord>(); // for temporary storing dayrecords, because must read from end to start.
		
		reader = new BufferedReader(new InputStreamReader(csv));
		
		try {
			reader.readLine(); // read titles
			String[] splittedRecord = splitRecord(reader.readLine()); // parse first datarecord
			
			for(int i = 1; splittedRecord != null && (i <= daysBack || daysBack <= 0); i++) {
				// add new dayrecord
				temp.add((new DayRecord(parseLocalDateFromString(splittedRecord[0]), Double.parseDouble(splittedRecord[1]), Double.parseDouble(splittedRecord[2]), Double.parseDouble(splittedRecord[3]), Double.parseDouble(splittedRecord[4]), Long.parseLong(splittedRecord[5]), Double.parseDouble(splittedRecord[6]))));
				splittedRecord = splitRecord(reader.readLine());
			}
			
			// add dayrecords from temp arrayList
			for(int i=temp.size()-1;i>=0;i--) {
		        equity.getDayrecords().add(temp.get(i));
		    }
		}
		catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	
	private static String[] splitRecord(String record) {
		if(record != null) {
			return record.split(",");			
		}
		return null;
	}
	
	
	
	private static LocalDate parseLocalDateFromString(String string) {
		String[] dateDetails = string.split("-");
		LocalDate ld = LocalDate.of(Integer.parseInt(dateDetails[0]), Integer.parseInt(dateDetails[1]), Integer.parseInt(dateDetails[2]));
		return	ld;
	}
}
