package ch.loosli.protrading.signalLoader;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;


/**
 * 
 * @author Roman Loosli
 * @version 1.0
 * @since Version 1.0
 *
 */
public class QuoteLoader implements Runnable{

	
	private static final String basicUrl = "http://ichart.finance.yahoo.com/table.csv";
	private static final String ticker = "?s=";
	private static final String startMonth = "?a="; // starting with 0 for january
	private static final String startDay = "?b="; // 1-31
	private static final String startYear = "?c";
	private static final String endMonth = "?d="; // starting with 0 for january
	private static final String endDay = "?e="; // 1-31
	private static final String endYear = "?f=";

	
	private static ArrayList<Equity> quoteList = new ArrayList<Equity>();
	
	
	public QuoteLoader() {
		
	}
	
	
	/**
	 * @return the quoteList
	 */
	public static ArrayList<Equity> getQuoteList() {
		return quoteList;
	}


	/**
	 * Loads all Dayrecords from a specified Ticker to its Equity.
	 * @param ticker Specified Ticker
	 * @return The Equity, on which the DayRecords were added.
	 */
	protected Equity loadByTicker(String ticker) {
		String url = prepareUrl(ticker, null, null);
		
		Equity e = null;
		
		InputStream input;
		try {
			input = new URL(url).openStream();
			e = new Equity(ticker);
			QuoteFileParser.addDayRecordsFromFile(e, input, 0);
			e.setSignal(SignalChooser.chooseSignal(e));
		} catch (IOException ioe) {
		}

		return e;
	}
	
	
	/**
	 * <p>Increases a tickersymbol.</p>
	 * </p>
	 * <p>Examples:</p>
	 * <p>'Z' -> 'AA'</br>
	 * 'RFZZ' -> 'RGAA'</br>
	 * 'HBZY' -> 'HBZZ'</p>
	 * @param oldTicker Ticker which should be increased.
	 * @return Increased Ticker.
	 */
	protected String nextTicker(String oldTicker) {
		return nextTicker(new StringBuilder(oldTicker)).toString();
	}
	
	
	
	/**
	 * <p>A counter for tickersymbols.</p>
	 * </p>
	 * <p>Examples:</p>
	 * <p>'Z' -> 'AA'</br>
	 * 'RFZZ' -> 'RGAA'</br>
	 * 'HBZY' -> 'HBZZ'</p>
	 * @param oldTicker Ticker which should be increased.
	 * @return Increased Ticker.
	 */
	protected StringBuilder nextTicker(StringBuilder oldTicker) {
		StringBuilder newTicker = new StringBuilder();

		for(byte checkedCharFromBack = 1; checkedCharFromBack <= oldTicker.length(); checkedCharFromBack++) {
			if(!(oldTicker.charAt(oldTicker.length() - checkedCharFromBack) == 'Z')) {
				char newChar = (char) (oldTicker.charAt(oldTicker.length() - checkedCharFromBack) + 1);
				
				// Not affected leading chars -> move to newTicker
				for(int i = 0; i < oldTicker.length() - checkedCharFromBack; i++) {
					newTicker.append(oldTicker.charAt(i));
				}
				
				// append new char and trailing 'A's
				newTicker.append(newChar);
				for(int i = newTicker.length(); i < oldTicker.length(); i++) {
					newTicker.append('A');
				}
				return newTicker;
			}
			
			// if every char in string equals 'Z', every char will be set to 'A' + a 'A' will be added.
			else if(oldTicker.charAt(oldTicker.length() - checkedCharFromBack) == 'Z' && oldTicker.length() - checkedCharFromBack == 0) {
				newTicker.append('A');
				for(int i = 1; i <= oldTicker.length(); i++) {
					newTicker.append('A');
				}
			}
		}
		return newTicker;
	}
	
	
	
	public ArrayList<Equity> searchThroughAllQuotes() {		

		do {
			Equity e = loadByTicker(TickerFilesParser.getInstance().getNextSP500Ticker());
			if(e != null) {
				quoteList.add(e);
			}
		} while(TickerFilesParser.getInstance().getNumReadSP500() % 500 != 0);
		
		return quoteList;
	}
	
	
	/**
	 * Takes basic URL and adds variables to execute URL request.
	 * @param ticker tickervariable
	 * @param startDate The variable, when the data in the requested CSV should start.
	 * @param endDate The variable, when the data in the requested CSV should end.
	 * @return The prepared URL for the data request.
	 */
	@SuppressWarnings({ "deprecation", "static-access" })
	private String prepareUrl(String ticker, Date startDate, Date endDate) {
		StringBuilder url = new StringBuilder(basicUrl);
		
		url.append(this.ticker);
		url.append(ticker);
		
		if(startDate != null) {
			url.append(this.startMonth);
			url.append(startDate.getMonth());
			url.append(this.startDay);
			url.append(startDate.getDate());
			url.append(this.startYear);
			url.append(startDate.getYear() + 1900);			
		}
		
		if(endDate != null) {
			url.append(this.endMonth);
			url.append(endDate.getMonth());
			url.append(this.endDay);
			url.append(endDate.getDate());
			url.append(this.endYear);
			url.append(endDate.getYear() + 1900);			
		}
		
		return url.toString();
	}


	@Override
	public void run() {
		searchThroughAllQuotes();
	}
}
